@@ -19,10 +19,11 @@ stan_predictor.btl <- function(x, ...) {
     stan_gp(x, ...),
     stan_ac(x, ...),
     stan_offset(x, ...),
-    stan_bhaz(x, ...),
-    stan_special_prior_global(x, ...)
+    stan_bhaz(x, ...)
   )
-  stan_eta_combine(out, bterms = x, ...)
+  out <- stan_special_prior(x, out = out, ...)
+  out <- stan_eta_combine(x, out = out, ...)
+  out
 }
 
 # prepare Stan code for non-linear terms
@@ -55,9 +56,7 @@ stan_predictor.brmsterms <- function(x, data, prior, normalize, ...) {
     dp_comment <- stan_dpar_comments(dp, family = x$family)
     if (is.btl(dp_terms) || is.btnl(dp_terms)) {
       # distributional parameter is predicted
-      inv_link <- stan_eta_inv_link(dp, bterms = x, resp = resp)
-      dp_args <- list(dp_terms, inv_link = inv_link)
-      str_add_list(out) <- do_call(stan_predictor, c(dp_args, args))
+      str_add_list(out) <- do_call(stan_predictor, c(list(dp_terms), args))
     } else if (is.numeric(x$fdpars[[dp]]$value)) {
       # distributional parameter is fixed to constant
       if (is_mix_proportion(dp, family = x$family)) {
@@ -100,7 +99,7 @@ stan_predictor.brmsterms <- function(x, data, prior, normalize, ...) {
         dp_comment <- paste0(dp_comment, " (temporary)")
       }
       str_add_list(out) <- stan_prior(
-        prior, dp, prefix = prefix, suffix = resp, 
+        prior, dp, prefix = prefix, suffix = resp,
         header_type = "real", px = px,
         comment = dp_comment, normalize = normalize
       )
@@ -127,7 +126,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
   resp_type <- out[[1]]$resp_type
   out <- collapse_lists(ls = out)
   out$resp_type <- "vector"
-  adforms <- lapply(x$terms, "[[", "adforms")
+  adforms <- from_list(x$terms, "adforms")
   adnames <- unique(ulapply(adforms, names))
   adallowed <- c("se", "weights", "mi")
   if (!all(adnames %in% adallowed))  {
@@ -167,7 +166,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
     )
     str_add(out$pll_args) <- glue(", data vector weights")
   }
-  miforms <- rmNULL(lapply(adforms, "[[", "mi"))
+  miforms <- rmNULL(from_list(adforms, "mi"))
   if (length(miforms)) {
     str_add(out$model_no_pll_def) <- "  vector[nresp] Yl[N] = Y;\n"
     str_add(out$pll_args) <- ", vector[] Yl"
@@ -188,7 +187,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
   )
   if (family == "student") {
     str_add_list(out) <- stan_prior(
-      prior, class = "nu", header_type = "real", 
+      prior, class = "nu", header_type = "real",
       normalize = normalize
     )
   }
@@ -284,6 +283,8 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
+  lpdf <- stan_lpdf_name(normalize)
+
   if (length(fixef)) {
     str_add(out$data) <- glue(
       "  int<lower=1> K{p};",
@@ -313,41 +314,33 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
     }
     # prepare population-level coefficients
     b_type <- glue("vector[K{ct}{p}]")
-    has_special_b_prior <- stan_has_special_b_prior(bterms, prior)
-    assign_b_tpar <- stan_assign_b_tpar(bterms, prior)
+    has_special_prior <- has_special_prior(prior, bterms, class = "b")
     if (decomp == "none") {
-      b_suffix <- ""
-      b_comment <- "population-level effects"
-      if (has_special_b_prior) {
-        stopif_prior_bound(prior, class = "b", ls = px)
-        if (assign_b_tpar) {
-          # only some special priors assign b in transformed parameters
-          str_add(out$tpar_def) <- glue("  {b_type} b{p};  // {b_comment}\n")
-          str_add(out$pll_args) <- glue(", vector b{p}")
-        }
+      if (has_special_prior) {
+        str_add_list(out) <- stan_prior_non_centered(
+          suffix = p, suffix_K = ct, normalize = normalize
+        )
       } else {
         str_add_list(out) <- stan_prior(
           prior, class = "b", coef = fixef, type = b_type,
-          px = px, suffix = p, header_type = "vector", 
-          comment = b_comment, normalize = normalize
+          px = px, suffix = p, header_type = "vector",
+          comment = "regression coefficients", normalize = normalize
         )
       }
     } else {
       stopifnot(decomp == "QR")
       stopif_prior_bound(prior, class = "b", ls = px)
-      b_suffix <- "Q"
-      b_comment <- "regression coefficients at QR scale"
-      if (has_special_b_prior) {
-        if (assign_b_tpar) {
-          # only some special priors assign b in transformed parameters
-          str_add(out$tpar_def) <- glue("  {b_type} bQ{p};  // {b_comment}\n")
-          str_add(out$pll_args) <- glue(", vector bQ{p}")
-        }
+      if (has_special_prior) {
+        str_add_list(out) <- stan_prior_non_centered(
+          suffix = p, suffix_class = "Q", suffix_K = ct,
+          normalize = normalize
+        )
       } else {
         str_add_list(out) <- stan_prior(
           prior, class = "b", coef = fixef, type = b_type,
-          px = px, suffix = glue("Q{p}"), header_type = "vector", 
-          comment = b_comment, normalize = normalize
+          px = px, suffix = glue("Q{p}"), header_type = "vector",
+          comment = "regression coefficients on QR scale",
+          normalize = normalize
         )
       }
       str_add(out$gen_def) <- glue(
@@ -355,11 +348,6 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
         "  vector[K{ct}{p}] b{p} = XR{p}_inv * bQ{p};\n"
       )
     }
-    str_add_list(out) <- stan_special_prior_local(
-      prior, class = "b", ncoef = length(fixef),
-      px = px, center_X = center_X, suffix = b_suffix,
-      normalize = normalize
-    )
   }
 
   order_intercepts <- order_intercepts(bterms)
@@ -374,11 +362,13 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
     # centering the design matrix improves convergence
     sub_X_means <- ""
     if (length(fixef)) {
+      str_add(out$data) <- glue(
+        "  int<lower=1> Kc{p};",
+        "  // number of population-level effects after centering\n"
+      )
       sub_X_means <- glue(" - dot_product(means_X{p}, b{p})")
       if (is_ordinal(family)) {
-        # the intercept was already removed during the data preparation
         str_add(out$tdata_def) <- glue(
-          "  int Kc{p} = K{p};\n",
           "  matrix[N{resp}, Kc{p}] Xc{p};",
           "  // centered version of X{p}\n",
           "  vector[Kc{p}] means_X{p};",
@@ -392,7 +382,6 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
         )
       } else {
         str_add(out$tdata_def) <- glue(
-          "  int Kc{p} = K{p} - 1;\n",
           "  matrix[N{resp}, Kc{p}] Xc{p};",
           "  // centered version of X{p} without an intercept\n",
           "  vector[Kc{p}] means_X{p};",
@@ -568,8 +557,14 @@ stan_re <- function(ranef, prior, normalize, ...) {
       )
     }
   }
+
   # define standard deviation parameters
+  has_special_prior <- has_special_prior(prior, px, class = "sd")
   if (has_by) {
+    if (has_special_prior) {
+      stop2("Special priors on class 'sd' are not yet compatible ",
+            "with the 'by' argument.")
+    }
     str_add_list(out) <- stan_prior(
       prior, class = "sd", group = r$group[1], coef = r$coef,
       type = glue("matrix[M_{id}, Nby_{id}]"),
@@ -579,13 +574,25 @@ stan_re <- function(ranef, prior, normalize, ...) {
       normalize = normalize
     )
   } else {
-    str_add_list(out) <- stan_prior(
-      prior, class = "sd", group = r$group[1], coef = r$coef,
-      type = glue("vector[M_{id}]"), suffix = glue("_{id}"), px = px,
-      comment = "group-level standard deviations",
-      normalize = normalize
-    )
+    if (has_special_prior) {
+      if (stan_has_multiple_base_priors(px)) {
+        stop2("Special priors on class 'sd' are not yet compatible with ",
+              "group-level coefficients correlated across formulas.")
+      }
+      str_add(out$tpar_def) <- glue(
+        "  vector<lower=0>[M_{id}] sd_{id};  // group-level standard deviations\n"
+      )
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sd", group = r$group[1], coef = r$coef,
+        type = glue("vector[M_{id}]"), suffix = glue("_{id}"), px = px,
+        comment = "group-level standard deviations",
+        normalize = normalize
+      )
+    }
   }
+
+  # define group-level coefficients
   dfm <- ""
   tr <- get_dist_groups(r, "student")
   if (nrow(r) > 1L && r$cor[1]) {
@@ -737,15 +744,24 @@ stan_sm <- function(bterms, data, prior, threads, normalize, ...) {
       "  // design matrix for the linear effects\n"
     )
     str_add(out$pll_args) <- glue(", data matrix Xs{p}")
-    str_add_list(out) <- stan_prior(
-      prior, class = "b", coef = Xs_names,
-      type = glue("vector[Ks{p}]"), suffix = glue("s{p}"),
-      header_type = "vector", px = px,
-      comment = "spline coefficients", normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "b")) {
+      str_add_list(out) <- stan_prior_non_centered(
+        suffix = glue("s{p}"), normalize = normalize
+      )
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "b", coef = Xs_names,
+        type = glue("vector[Ks{p}]"), suffix = glue("s{p}"),
+        header_type = "vector", px = px,
+        comment = "spline coefficients", normalize = normalize
+      )
+    }
     str_add(out$eta) <- glue(" + Xs{p}{slice} * bs{p}")
   }
   for (i in seq_rows(smef)) {
+    if (smef$nbases[[i]] == 0) {
+      next  # no penalized spline components present
+    }
     pi <- glue("{p}_{i}")
     nb <- seq_len(smef$nbases[[i]])
     str_add(out$data) <- glue(
@@ -765,11 +781,17 @@ stan_sm <- function(bterms, data, prior, threads, normalize, ...) {
       "  // standarized spline coefficients\n",
       "  vector[knots{pi}[{nb}]] zs{pi}_{nb};\n"
     )
-    for (j in nb) {
+    if (has_special_prior(prior, px, class = "sds")) {
+      str_add(out$tpar_def) <- glue(
+        "  vector<lower=0>[nb{pi}] sds{pi};  // SDs of spline coefficients\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sds{pi}")
+      str_add(out$prior_global_lengths) <- glue(" nb{pi}")
+    } else {
       str_add_list(out) <- stan_prior(
-        prior, class = "sds", coef = smef$term[i],
-        suffix = glue("{pi}_{j}"), px = px,
-        comment = "standard deviations of spline coefficients",
+        prior, class = "sds", coef = smef$term[i], suffix = pi, px = px,
+        type = glue("vector[nb{pi}]"), coef_type = glue("vector[nb{pi}]"),
+        comment = "SDs of spline coefficients",
         normalize = normalize
       )
     }
@@ -778,9 +800,9 @@ stan_sm <- function(bterms, data, prior, threads, normalize, ...) {
       "  // actual spline coefficients\n",
       "  vector[knots{pi}[{nb}]] s{pi}_{nb};\n"
     )
-    str_add(out$tpar_comp) <- cglue(
+    str_add(out$tpar_special_prior) <- cglue(
       "  // compute actual spline coefficients\n",
-      "  s{pi}_{nb} = sds{pi}_{nb} * zs{pi}_{nb};\n"
+      "  s{pi}_{nb} = sds{pi}[{nb}] * zs{pi}_{nb};\n"
     )
     str_add(out$pll_args) <- cglue(", vector s{pi}_{nb}")
     str_add(out$model_prior) <- cglue(
@@ -866,10 +888,13 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
                     normalize, ...) {
   out <- list()
   spef <- tidy_spef(bterms, data)
-  if (!nrow(spef)) return(out)
+  if (!nrow(spef)) {
+    return(out)
+  }
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
+  lpdf <- stan_lpdf_name(normalize)
   n <- stan_nn(threads)
   ranef <- subset2(ranef, type = "sp", ls = px)
   spef_coef <- rename(spef$term)
@@ -978,15 +1003,11 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
   }
 
   # prepare special effects coefficients
-  if (stan_has_special_b_prior(bterms, prior)) {
+  if (has_special_prior(prior, bterms, class = "b")) {
     stopif_prior_bound(prior, class = "b", ls = px)
-    if (stan_assign_b_tpar(bterms, prior)) {
-      # only some special priors assign b in transformed parameters
-      str_add(out$tpar_def) <- glue(
-        "  // special effects coefficients\n",
-        "  vector[Ksp{p}] bsp{p};\n"
-      )
-    }
+    str_add_list(out) <- stan_prior_non_centered(
+      suffix = glue("sp{p}"), normalize = normalize
+    )
   } else {
     str_add_list(out) <- stan_prior(
       prior, class = "b", coef = spef$coef,
@@ -996,11 +1017,6 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
       normalize = normalize
     )
   }
-  stan_special_priors <- stan_special_prior_local(
-    prior, class = "bsp", ncoef = nrow(spef),
-    px = px, center_X = FALSE, normalize = normalize
-  )
-  out <- collapse_lists(out, stan_special_priors)
   out
 }
 
@@ -1038,12 +1054,20 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
         "  int<lower=1> NBgp{pi};\n"
       )
     }
-    str_add_list(out) <- stan_prior(
-      prior, class = "sdgp", coef = sfx1,
-      type = glue("vector[Kgp{pi}]"), px = px, suffix = pi,
-      comment = "GP standard deviation parameters",
-      normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "sdgp")) {
+      str_add(out$tpar_def) <- glue(
+        "  vector<lower=0>[Kgp{pi}] sdgp{pi};  // GP standard deviation parameters\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sdgp{pi}")
+      str_add(out$prior_global_lengths) <- glue(" Kgp{pi}")
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sdgp", coef = sfx1, px = px, suffix = pi,
+        type = glue("vector[Kgp{pi}]"), coef_type = glue("vector[Kgp{pi}]"),
+        comment = "GP standard deviation parameters",
+        normalize = normalize
+      )
+    }
     if (gpef$iso[i]) {
       lscale_type <- "vector[1]"
       lscale_dim <- glue("[Kgp{pi}]")
@@ -1073,7 +1097,7 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
       )
       str_add_list(out) <- stan_prior(
         prior, class = "lscale", coef = sfx2,
-        type = lscale_type, dim = lscale_dim, suffix = glue("{pi}"), 
+        type = lscale_type, dim = lscale_dim, suffix = glue("{pi}"),
         px = px, comment = lscale_comment, normalize = normalize
       )
       if (gr) {
@@ -1237,7 +1261,7 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
   slice <- stan_slice(threads)
   has_natural_residuals <- has_natural_residuals(bterms)
   has_ac_latent_residuals <- has_ac_latent_residuals(bterms)
-  acef <- tidy_acef(bterms, data)
+  acef <- tidy_acef(bterms)
 
   if (has_ac_latent_residuals) {
     # families that do not have natural residuals require latent
@@ -1249,10 +1273,18 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
     str_add(out$par) <- glue(
       "  vector[N{resp}] zerr{p};  // unscaled residuals\n"
     )
-    str_add_list(out) <- stan_prior(
-      prior, class = "sderr", px = px, suffix = p,
-      comment = "SD of residuals", normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "sderr")) {
+      str_add(out$tpar_def) <- glue(
+        "  real<lower=0> sderr{p};  // SD of residuals\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sderr{p}")
+      str_add(out$prior_global_lengths) <- glue(" 1")
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sderr", px = px, suffix = p,
+        comment = "SD of residuals", normalize = normalize
+      )
+    }
     str_add(out$tpar_def) <- glue(
       "  vector[N{resp}] err{p};  // actual residuals\n"
     )
@@ -1329,24 +1361,42 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
       )
     }
     if (acef_arma$p > 0) {
-      str_add_list(out) <- stan_prior(
-        prior, class = "ar", px = px, suffix = p,
-        coef = seq_along(acef_arma$p),
-        type = glue("vector[Kar{p}]"),
-        header_type = "vector",
-        comment = "autoregressive coefficients",
-        normalize = normalize
-      )
+      if (has_special_prior(prior, px, class = "ar")) {
+        if (acef_arma$cov) {
+          stop2("Cannot use shrinkage priors on 'ar' if cov = TRUE.")
+        }
+        str_add_list(out) <- stan_prior_non_centered(
+          class = "ar", suffix = p, suffix_K = "ar"
+        )
+      } else {
+        str_add_list(out) <- stan_prior(
+          prior, class = "ar", px = px, suffix = p,
+          coef = seq_along(acef_arma$p),
+          type = glue("vector[Kar{p}]"),
+          header_type = "vector",
+          comment = "autoregressive coefficients",
+          normalize = normalize
+        )
+      }
     }
     if (acef_arma$q > 0) {
-      str_add_list(out) <- stan_prior(
-        prior, class = "ma", px = px, suffix = p,
-        coef = seq_along(acef_arma$q),
-        type = glue("vector[Kma{p}]"),
-        header_type = "vector",
-        comment = "moving-average coefficients",
-        normalize = normalize
-      )
+      if (has_special_prior(prior, px, class = "ma")) {
+        if (acef_arma$cov) {
+          stop2("Cannot use shrinkage priors on 'ma' if cov = TRUE.")
+        }
+        str_add_list(out) <- stan_prior_non_centered(
+          class = "ma", suffix = p, suffix_K = "ma"
+        )
+      } else {
+        str_add_list(out) <- stan_prior(
+          prior, class = "ma", px = px, suffix = p,
+          coef = seq_along(acef_arma$q),
+          type = glue("vector[Kma{p}]"),
+          header_type = "vector",
+          comment = "moving-average coefficients",
+          normalize = normalize
+        )
+      }
     }
   }
 
@@ -1361,12 +1411,30 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
     )
   }
 
+  acef_unstr <- subset2(acef, class = "unstr")
+  if (NROW(acef_unstr)) {
+    # unstructured correlation matrix
+    # most code is shared with ARMA covariance models
+    # define prior on the Cholesky scale to consistency across
+    # autocorrelation structures
+    str_add_list(out) <- stan_prior(
+      prior, class = "Lcortime", px = px, suffix = p,
+      type = glue("cholesky_factor_corr[n_unique_t{p}]"),
+      header_type = "matrix",
+      comment = "cholesky factor of unstructured autocorrelation matrix",
+      normalize = normalize
+    )
+  }
+
   acef_time_cov <- subset2(acef, dim = "time", cov = TRUE)
   if (NROW(acef_time_cov)) {
     # use correlation structures in covariance matrix parameterization
-    # optional for ARMA models and obligatory for COSY models
+    # optional for ARMA models and obligatory for COSY and UNSTR models
     # can only model one covariance structure at a time
     stopifnot(NROW(acef_time_cov) == 1)
+    if (use_threading(threads)) {
+      stop2("Threading is not supported for covariance-based autocorrelation models.")
+    }
     str_add(out$data) <- glue(
       "  // see the functions block for details\n",
       "  int<lower=1> N_tg{p};\n",
@@ -1374,46 +1442,75 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
       "  int<lower=1> end_tg{p}[N_tg{p}];\n",
       "  int<lower=1> nobs_tg{p}[N_tg{p}];\n"
     )
+    str_add(out$pll_args) <- glue(
+      ", int[] begin_tg{p}, int[] end_tg{p}, int[] nobs_tg{p}"
+    )
     str_add(out$tdata_def) <- glue(
       "  int max_nobs_tg{p} = max(nobs_tg{p});",
       "  // maximum dimension of the autocorrelation matrix\n"
     )
-    if (!is.formula(bterms$adforms$se)) {
-      str_add(out$tdata_def) <- glue(
-        "  // no known standard errors specified by the user\n",
-        "  vector[N{resp}] se2{p} = rep_vector(0.0, N{resp});\n"
+    if (acef_time_cov$class == "unstr") {
+      # unstructured time-covariances require additional data and cannot
+      # be represented directly via Cholesky factors due to potentially
+      # different time subsets
+      str_add(out$data) <- glue(
+        "  int<lower=0> Jtime_tg{p}[N_tg{p}, max(nobs_tg{p})];\n",
+        "  int n_unique_t{p};  // total number of unique time points\n",
+        "  int n_unique_cortime{p};  // number of unique correlations\n"
       )
-    }
-    str_add(out$tpar_def) <- glue(
-      "  // cholesky factor of the autocorrelation matrix\n",
-      "  matrix[max_nobs_tg{p}, max_nobs_tg{p}] chol_cor{p};\n"
-    )
-    if (acef_time_cov$class == "arma") {
-      if (acef_time_cov$p > 0 && acef_time_cov$q == 0) {
-        cor_fun <- "ar1"
-        cor_args <- glue("ar{p}[1]")
-      } else if (acef_time_cov$p == 0 && acef_time_cov$q > 0) {
-        cor_fun <- "ma1"
-        cor_args <- glue("ma{p}[1]")
-      } else {
-        cor_fun <- "arma1"
-        cor_args <- glue("ar{p}[1], ma{p}[1]")
+      str_add(out$pll_args) <- glue(", int[,] Jtime_tg{p}")
+      if (has_ac_latent_residuals) {
+        str_add(out$tpar_comp) <- glue(
+          "  // compute correlated time-series residuals\n",
+          "  err{p} = scale_time_err_flex(",
+          "zerr{p}, sderr{p}, Lcortime{p}, nobs_tg{p}, begin_tg{p}, end_tg{p}, Jtime_tg{p});\n"
+        )
       }
-    } else if (acef_time_cov$class == "cosy") {
-      cor_fun <- "cosy"
-      cor_args <- glue("cosy{p}")
-    }
-    str_add(out$tpar_comp) <- glue(
-      "  // compute residual covariance matrix\n",
-      "  chol_cor{p} = cholesky_cor_{cor_fun}({cor_args}, max_nobs_tg{p});\n"
-    )
-    if (has_ac_latent_residuals) {
-      str_add(out$tpar_comp) <- glue(
-        "  // compute correlated time-series residuals\n",
-        "  err{p} = scale_time_err(",
-        "zerr{p}, sderr{p}, chol_cor{p}, nobs_tg{p}, begin_tg{p}, end_tg{p});\n"
+      str_add(out$gen_def) <- glue(
+        "  // compute group-level correlations\n",
+        "  corr_matrix[n_unique_t{p}] Cortime{p}",
+        " = multiply_lower_tri_self_transpose(Lcortime{p});\n",
+        "  vector<lower=-1,upper=1>[n_unique_cortime{p}] cortime{p};\n"
       )
+      str_add(out$gen_comp) <- stan_cor_gen_comp(
+        glue("cortime{p}"), glue("n_unique_t{p}")
+      )
+    } else {
+      # all other time-covariance structures can be represented directly
+      # through Cholesky factors of the correlation matrix
+      if (acef_time_cov$class == "arma") {
+        if (acef_time_cov$p > 0 && acef_time_cov$q == 0) {
+          cor_fun <- "ar1"
+          cor_args <- glue("ar{p}[1]")
+        } else if (acef_time_cov$p == 0 && acef_time_cov$q > 0) {
+          cor_fun <- "ma1"
+          cor_args <- glue("ma{p}[1]")
+        } else {
+          cor_fun <- "arma1"
+          cor_args <- glue("ar{p}[1], ma{p}[1]")
+        }
+      } else if (acef_time_cov$class == "cosy") {
+        cor_fun <- "cosy"
+        cor_args <- glue("cosy{p}")
+      }
+      str_add(out$tpar_def) <- glue(
+        "  // cholesky factor of the autocorrelation matrix\n",
+        "  matrix[max_nobs_tg{p}, max_nobs_tg{p}] Lcortime{p};\n"
+      )
+      str_add(out$pll_args) <- glue(", matrix Lcortime{p}")
+      str_add(out$tpar_comp) <- glue(
+        "  // compute residual covariance matrix\n",
+        "  Lcortime{p} = cholesky_cor_{cor_fun}({cor_args}, max_nobs_tg{p});\n"
+      )
+      if (has_ac_latent_residuals) {
+        str_add(out$tpar_comp) <- glue(
+          "  // compute correlated time-series residuals\n",
+          "  err{p} = scale_time_err(",
+          "zerr{p}, sderr{p}, Lcortime{p}, nobs_tg{p}, begin_tg{p}, end_tg{p});\n"
+        )
+      }
     }
+
   }
 
   acef_sar <- subset2(acef, class = "sar")
@@ -1461,10 +1558,18 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
       "  int<lower=1> edges1{p}[Nedges{p}];\n",
       "  int<lower=1> edges2{p}[Nedges{p}];\n"
     )
-    str_add_list(out) <- stan_prior(
-      prior, class = "sdcar", px = px, suffix = p,
-      comment = "SD of the CAR structure", normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "sdcar")) {
+      str_add(out$tpar_def) <- glue(
+        "  real<lower=0> sdcar{p};  // SD of the CAR structure\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sdcar{p}")
+      str_add(out$prior_global_lengths) <- glue(" 1")
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sdcar", px = px, suffix = p,
+        comment = "SD of the CAR structure", normalize = normalize
+      )
+    }
     str_add(out$pll_args) <- glue(", vector rcar{p}, data int[] Jloc{p}")
     str_add(out$loopeta) <- glue(" + rcar{p}[Jloc{p}{n}]")
     if (acef_car$type %in% c("escar", "esicar")) {
@@ -1608,9 +1713,7 @@ stan_offset <- function(bterms, threads, ...) {
 
 # Stan code for non-linear predictor terms
 # @param nlpars names of the non-linear parameters
-# @param inv_link character vector of length 2 defining the link to be applied
-stan_nl <- function(bterms, data, nlpars, threads, inv_link = rep("", 2), ...) {
-  stopifnot(length(inv_link) == 2L)
+stan_nl <- function(bterms, data, nlpars, threads, ...) {
   out <- list()
   resp <- usc(bterms$resp)
   par <- combine_prefix(bterms, keep_mu = TRUE, nlp = TRUE)
@@ -1630,19 +1733,36 @@ stan_nl <- function(bterms, data, nlpars, threads, inv_link = rep("", 2), ...) {
       slice <- stan_slice(threads)
     }
     slice <- paste0(slice, " ")
-    str_add(out$data) <- "  // covariate vectors for non-linear functions\n"
+    str_add(out$data) <- "  // covariates for non-linear functions\n"
     for (i in seq_along(covars)) {
-      is_integer <- is.integer(data_cnl[[glue("C{p}_{i}")]])
+      cname <- glue("C{p}_{i}")
+      is_integer <- is.integer(data_cnl[[cname]])
+      is_matrix <- is.matrix(data_cnl[[cname]])
+      dim2 <- dim(data_cnl[[cname]])[2]
       if (is_integer) {
-        str_add(out$data) <- glue(
-          "  int C{p}_{i}[N{resp}];\n"
-        )
-        str_add(out$pll_args) <- glue(", data int[] C{p}_{i}")
+        if (is_matrix) {
+          str_add(out$data) <- glue(
+            "  int C{p}_{i}[N{resp}, {dim2}];\n"
+          )
+          str_add(out$pll_args) <- glue(", data int[,] C{p}_{i}")
+        } else {
+          str_add(out$data) <- glue(
+            "  int C{p}_{i}[N{resp}];\n"
+          )
+          str_add(out$pll_args) <- glue(", data int[] C{p}_{i}")
+        }
       } else {
-        str_add(out$data) <- glue(
-          "  vector[N{resp}] C{p}_{i};\n"
-        )
-        str_add(out$pll_args) <- glue(", data vector C{p}_{i}")
+        if (is_matrix) {
+          str_add(out$data) <- glue(
+            "  matrix[N{resp}, {dim2}] C{p}_{i};\n"
+          )
+          str_add(out$pll_args) <- glue(", data matrix C{p}_{i}")
+        } else {
+          str_add(out$data) <- glue(
+            "  vector[N{resp}] C{p}_{i};\n"
+          )
+          str_add(out$pll_args) <- glue(", data vector C{p}_{i}")
+        }
       }
       new_covars[i] <- glue(" C{p}_{i}{slice}")
     }
@@ -1653,7 +1773,7 @@ stan_nl <- function(bterms, data, nlpars, threads, inv_link = rep("", 2), ...) {
     ",", "==", "!=", "<=", ">=", "<", ">", "!", "&&", "||"
   )
   regex <- glue("(?<!\\.){escape_all(syms)}(?!=)")
-  eta <- rm_wsp(collapse(deparse(bterms$formula[[2]])))
+  eta <- rm_wsp(deparse0(bterms$formula[[2]]))
   eta <- wsp(rename(eta, regex, wsp(syms), fixed = FALSE, perl = TRUE))
   vars <- c(wsp(nlpars), wsp(covars), " ( ", " ) ")
   new_vars <- c(new_nlpars, new_covars, "(", ")")
@@ -1663,21 +1783,26 @@ stan_nl <- function(bterms, data, nlpars, threads, inv_link = rep("", 2), ...) {
     "  // initialize non-linear predictor term\n",
     "  vector[N{resp}] {par};\n"
   )
-  # make sure mu comes last as it might depend on other parameters
-  is_mu <- isTRUE("mu" %in% dpar_class(bterms[["dpar"]]))
-  position <- str_if(is_mu, "model_comp_mu_link", "model_comp_dpar_link")
   if (bterms$loop) {
-    str_add(out[[position]]) <- glue(
+    inv_link <- stan_inv_link(
+      bterms$family$link, vectorize = FALSE,
+      transform = bterms$transform
+    )
+    str_add(out$model_comp_dpar_link) <- glue(
       "  for (n in 1:N{resp}) {{\n",
       stan_nn_def(threads),
       "    // compute non-linear predictor values\n",
-      "    {par}[n] = {inv_link[1]}{eta}{inv_link[2]};\n",
+      "    {par}[n] = {inv_link}({eta});\n",
       "  }}\n"
     )
   } else {
-    str_add(out[[position]]) <- glue(
+    inv_link <- stan_inv_link(
+      bterms$family$link, vectorize = TRUE,
+      transform = bterms$transform
+    )
+    str_add(out$model_comp_dpar_link) <- glue(
       "  // compute non-linear predictor values\n",
-      "  {par} = {inv_link[1]}{eta}{inv_link[2]};\n"
+      "  {par} = {inv_link}({eta});\n"
     )
   }
   out
@@ -1799,12 +1924,10 @@ stan_Xme <- function(meef, prior, threads, normalize) {
 # @param bterms btl object
 # @param ranef output of tidy_ranef
 # @param primitive use Stan's GLM likelihood primitives?
-# @param inv_link character vector of length 2 defining the link to be applied
 # @param ... currently unused
 # @return list of character strings containing Stan code
-stan_eta_combine <- function(out, bterms, ranef, threads, primitive,
-                             inv_link = c("", ""), ...) {
-  stopifnot(is.list(out), is.btl(bterms), length(inv_link) == 2L)
+stan_eta_combine <- function(bterms, out, ranef, threads, primitive, ...) {
+  stopifnot(is.btl(bterms), is.list(out))
   if (primitive && !has_special_terms(bterms)) {
     # only overall effects and perhaps an intercept are present
     # which will be evaluated directly in the GLM primitive likelihood
@@ -1816,8 +1939,11 @@ stan_eta_combine <- function(out, bterms, ranef, threads, primitive,
   out$eta <- sub("^[ \t\r\n]+\\+", "", out$eta, perl = TRUE)
   str_add(out$model_def) <- glue(
     "  // initialize linear predictor term\n",
-    "  vector[N{resp}] {eta} ={out$eta};\n"
+    "  vector[N{resp}] {eta} = rep_vector(0.0, N{resp});\n"
   )
+  if (nzchar(out$eta)) {
+    str_add(out$model_comp_eta) <- glue("  {eta} +={out$eta};\n")
+  }
   out$eta <- NULL
   str_add(out$loopeta) <- stan_eta_re(ranef, threads = threads, px = px)
   if (nzchar(out$loopeta)) {
@@ -1833,15 +1959,13 @@ stan_eta_combine <- function(out, bterms, ranef, threads, primitive,
   }
   out$loopeta <- NULL
   # possibly transform eta before it is passed to the likelihood
-  if (sum(nzchar(inv_link))) {
-    # make sure mu comes last as it might depend on other parameters
-    is_mu <- isTRUE("mu" %in% dpar_class(bterms[["dpar"]]))
-    position <- str_if(is_mu, "model_comp_mu_link", "model_comp_dpar_link")
-    str_add(out[[position]]) <- glue(
-      "  for (n in 1:N{resp}) {{\n",
-      "    // apply the inverse link function\n",
-      "    {eta}[n] = {inv_link[1]}{eta}[n]{inv_link[2]};\n",
-      "  }}\n"
+  inv_link <- stan_inv_link(
+    bterms$family$link, vectorize = TRUE,
+    transform = bterms$transform
+  )
+  if (nzchar(inv_link)) {
+    str_add(out$model_comp_dpar_link) <- glue(
+      "  {eta} = {inv_link}({eta});\n"
     )
   }
   out
@@ -1853,33 +1977,31 @@ stan_eta_combine <- function(out, bterms, ranef, threads, primitive,
 # @param primitive use Stan's GLM likelihood primitives?
 # @return a single character string
 stan_eta_fe <- function(fixef, bterms, threads, primitive) {
-  if (length(fixef) && !primitive) {
-    p <- usc(combine_prefix(bterms))
-    center_X <- stan_center_X(bterms)
-    decomp <- get_decomp(bterms$fe)
-    sparse <- is_sparse(bterms$fe)
-    if (sparse) {
-      stopifnot(!center_X && decomp == "none")
-      csr_args <- sargs(
-        paste0(c("rows", "cols"), "(X", p, ")"),
-        paste0(c("wX", "vX", "uX", "b"), p)
-      )
-      eta_fe <- glue("csr_matrix_times_vector({csr_args})")
-    } else {
-      sfx_X <- sfx_b <- ""
-      if (decomp == "QR") {
-        sfx_X <- sfx_b <- "Q"
-      } else if (center_X) {
-        sfx_X <- "c"
-      }
-      slice <- stan_slice(threads)
-      eta_fe <- glue("X{sfx_X}{p}{slice} * b{sfx_b}{p}")
-    }
-  } else {
-    resp <- usc(bterms$resp)
-    eta_fe <- glue("rep_vector(0.0, N{resp})")
+  if (!length(fixef) || primitive) {
+    return("")
   }
-  glue(" + {eta_fe}")
+  p <- usc(combine_prefix(bterms))
+  center_X <- stan_center_X(bterms)
+  decomp <- get_decomp(bterms$fe)
+  sparse <- is_sparse(bterms$fe)
+  if (sparse) {
+    stopifnot(!center_X && decomp == "none")
+    csr_args <- sargs(
+      paste0(c("rows", "cols"), "(X", p, ")"),
+      paste0(c("wX", "vX", "uX", "b"), p)
+    )
+    eta_fe <- glue(" + csr_matrix_times_vector({csr_args})")
+  } else {
+    sfx_X <- sfx_b <- ""
+    if (decomp == "QR") {
+      sfx_X <- sfx_b <- "Q"
+    } else if (center_X) {
+      sfx_X <- "c"
+    }
+    slice <- stan_slice(threads)
+    eta_fe <- glue(" + X{sfx_X}{p}{slice} * b{sfx_b}{p}")
+  }
+  eta_fe
 }
 
 # write the group-level part of the linear predictor
@@ -1934,58 +2056,13 @@ stan_eta_rsp <- function(r) {
   out
 }
 
-# does eta need to be transformed manually using the link functions
-# @param family the model family
-# @param cens_or_trunc is the model censored or truncated?
+# does eta need to be transformed manually using the inv_link function
 stan_eta_transform <- function(family, bterms) {
-  transeta <- "transeta" %in% family_info(family, "specials")
-  no_transform <- family$link == "identity" && !transeta ||
+  no_transform <- family$link == "identity" ||
     has_joint_link(family) && !is.customfamily(family)
   !no_transform && !stan_has_built_in_fun(family, bterms)
 }
 
-# correctly apply inverse link to eta
-# @param dpar name of the parameter for which to define the link
-# @param bterms object of class 'brmsterms'
-# @param resp name of the response variable
-# @return a single character string
-stan_eta_inv_link <- function(dpar, bterms, resp = "") {
-  stopifnot(is.brmsterms(bterms))
-  out <- rep("", 2)
-  family <- bterms$dpars[[dpar]]$family
-  if (stan_eta_transform(family, bterms)) {
-    dpar_id <- dpar_id(dpar)
-    pred_dpars <- names(bterms$dpars)
-    shape <- glue("shape{dpar_id}")
-    n_shape <- str_if(shape %in% pred_dpars, "[n]")
-    shape <- glue("{shape}{resp}{n_shape}")
-    nu <- glue("nu{dpar_id}")
-    n_nu <- str_if(nu %in% pred_dpars, "[n]")
-    nu <- glue("{nu}{resp}{n_nu}")
-
-    family_link <- str_if(
-      family$family %in% c("gamma", "hurdle_gamma", "exponential"),
-      paste0(family$family, "_", family$link), family$family
-    )
-    inv_link <- stan_inv_link(family$link)
-    out <- switch(family_link,
-      c(glue("{inv_link}("), ")"),
-      gamma_log = c(glue("{shape} * exp(-("), "))"),
-      gamma_inverse = c(glue("{shape} * ("), ")"),
-      gamma_identity = c(glue("{shape} / ("), ")"),
-      hurdle_gamma_log = c(glue("{shape} * exp(-("), "))"),
-      hurdle_gamma_inverse = c(glue("{shape} * ("), ")"),
-      hurdle_gamma_identity = c(glue("{shape} / ("), ")"),
-      exponential_log = c("exp(-(", "))"),
-      exponential_inverse = c("(", ")"),
-      exponential_identity = c("inv(", ")"),
-      weibull = c(glue("{inv_link}("), glue(") / tgamma(1 + 1 / {shape})")),
-      frechet = c(glue("{inv_link}("), glue(") / tgamma(1 - 1 / {nu})"))
-    )
-  }
-  out
-}
-
 # indicate if the population-level design matrix should be centered
 # implies a temporary shift in the intercept of the model
 stan_center_X <- function(x) {
@@ -1993,20 +2070,6 @@ stan_center_X <- function(x) {
     !fix_intercepts(x) && !is_sparse(x$fe) && !has_sum_to_zero_thres(x)
 }
 
-# indicate if the overall coefficients 'b' have a special prior
-stan_has_special_b_prior <- function(bterms, prior) {
-  special <- get_special_prior(prior, bterms)
-  !is.null(special$horseshoe) || !is.null(special$R2D2) ||
-    !is.null(special$lasso)
-}
-
-# indicate if the overall coefficients 'b' should be
-# assigned in the transformed parameters block
-stan_assign_b_tpar <- function(bterms, prior) {
-  special <- get_special_prior(prior, bterms)
-  !is.null(special$horseshoe) || !is.null(special$R2D2)
-}
-
 stan_dpar_comments <- function(dpar, family) {
   dpar_class <- dpar_class(dpar, family)
   out <- switch(dpar_class, "",
@@ -2032,6 +2095,8 @@ stan_dpar_comments <- function(dpar, family) {
 }
 
 # Stan code for transformations of distributional parameters
+# TODO: refactor into family-specific functions
+# TODO: add gamma and related families here to compute rate = shape / mean
 stan_dpar_transform <- function(bterms, prior, threads, normalize, ...) {
   stopifnot(is.brmsterms(bterms))
   out <- list()
@@ -2108,12 +2173,12 @@ stan_dpar_transform <- function(bterms, prior, threads, normalize, ...) {
       xi <- glue("xi{id}")
       if (!xi %in% dp_names) {
         str_add(out$model_def) <- glue(
-          "  real {xi};  // scaled shape parameter\n"
+          "  real {xi}{p};  // scaled shape parameter\n"
         )
         sigma <- glue("sigma{id}")
         sfx <- str_if(sigma %in% names(bterms$dpars), "_vector")
         args <- sargs(
-          glue("tmp_{xi}"), glue("Y{p}"),
+          glue("tmp_{xi}{p}"), glue("Y{p}"),
           glue("mu{id}{p}"), glue("{sigma}{p}")
         )
         str_add(out$model_comp_dpar_trans) <- glue(
@@ -2124,7 +2189,7 @@ stan_dpar_transform <- function(bterms, prior, threads, normalize, ...) {
   }
   if (any(families %in% "logistic_normal")) {
     stopifnot(length(families) == 1L)
-    predcats <- get_predcats(bterms$family)
+    predcats <- make_stan_names(get_predcats(bterms$family))
     sigma_dpars <- glue("sigma{predcats}")
     reqn <- sigma_dpars %in% names(bterms$dpars)
     n <- ifelse(reqn, "[n]", "")
